/*
 * checks.h
 * Validate the given item is of the correct type.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>
 */

#ifndef __checks_h
#define	__checks_h

#include "debug.h"

#define	check_local_int(l)	DBGPRINT(DBG_CHECKS, ("- local[%d] int %d\n", (l), local(l)->v.tint))
#define	check_local_long(l)	DBGPRINT(DBG_CHECKS, ("- local[%d] long %ld\n", (l), (long int) local_long(l)->v.tlong))
#define	check_local_float(l)	DBGPRINT(DBG_CHECKS, ("- local[%d] float %g\n", (l), local_float(l)->v.tfloat))
#define	check_local_double(l)	DBGPRINT(DBG_CHECKS, ("- local[%d] double %Lg\n", (l), (long double) local_double(l)->v.tdouble))
#define	check_local_ref(l)	DBGPRINT(DBG_CHECKS, ("- local[%d] ref %p\n", (l), local(l)->v.taddr))

#define	check_stack_int(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] int %d\n", (l), stack(l)->v.tint))
#define	check_stack_long(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] long %ld\n", (l), (long int) stack(l)->v.tlong))
#define	check_stack_float(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] float %g\n", (l), stack(l)->v.tfloat))
#define	check_stack_double(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] double %Lg\n", (l), (long double) stack(l)->v.tdouble))
#define	check_stack_ref(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] ref %p\n", (l), stack(l)->v.taddr))

#define	check_stack_array(l)		DBGPRINT(DBG_CHECKS, ("- stack[%d] array * %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_intarray(l)		DBGPRINT(DBG_CHECKS, ("- stack[%d] array int %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_longarray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array long %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_floatarray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array float %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_doublearray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array double %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_refarray(l)		DBGPRINT(DBG_CHECKS, ("- stack[%d] array ref %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_bytearray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array byte %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_chararray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array char %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))
#define	check_stack_shortarray(l)	DBGPRINT(DBG_CHECKS, ("- stack[%d] array short %p[%d]\n", (l), stack(l)->v.taddr, *(uint32*)(object_array_length+(char*)stack(l)->v.taddr)))

/*
 * The following check_ macros all display the value of data embedded
 * in an instruction.  For example, the 'invokeinterface' instruction
 * has a 5 byte format.  The first byte is 0xb9 ('invokeinterface');
 * bytes 2 and 3 combine to be a 16bit index into the class constant
 * pool; byte 4 is the number of arg which should be on the stack
 * for this interface method;  byte 5 is zero;
 */

/*
 * Show value of a one-byte embedded operand at offset 'l' in embedded
 * operands.  (Byte -1 is the instruction id.)
 */
#define check_pc(l)		DBGPRINT(DBG_CHECKS, ("  code[%d] %d\n", l+1, getpc(l)))

/* Show target instruction of a wide instruction.
 * XXX never used.
 */
#define check_pc_wide(l)	DBGPRINT(DBG_CHECKS, ("  code[%d] %d (= %d + %d(wide))\n", l+1, getpc(l)+wide, getpc(l), wide))

/*
 * Show value of 'idx' part of instruction.
 * (eg 'ifnul' instruction is 3 bytes, 1st byte is
 * the 'ifnul' instruction and next two bytes are index, and
 * in the case of ifnul, the 2nd and 3rd bytes are the index
 * to jump to.
 * Looks at byte 'l' and 'l+1' in code.  (where -1 is the instruction id)
 */
#define check_pcidx(l)	DBGPRINT(DBG_CHECKS, ("  code[%d-%d] %d\n", l+1, l+2, (int16)((getpc(l) << 8) | getpc(l+1))))

/* Show a wide (32-bit) index, just as above. */
#define check_pcwidx(l)	DBGPRINT(DBG_CHECKS, ("  codew[%d-%d] %d\n", l+1, l+4, (int32)((getpc(l) << 24) | (getpc(l+1) << 16) | (getpc(l+2) << 8) | getpc(l+3))))

#endif
