/*
 * thread-kore.c
 * Kore library-specific thread support.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Patrick Tullmann <tullmann@cs.utah.edu>, June 1997
 */

/*
 * NOTE: This file is #included by thread.c
 */

/*
 * Initialise threads.
 */
void
initThreads(void)
{
DBG(DBG_SETUP,
    	printf("initThreads()\n");					)

	/* Set default thread stack size if not set */
	if (threadStackSize == 0) {
		threadStackSize = THREADSTACKSIZE;
	}

	/* Get a handle on the thread class */
	ThreadClass = lookupClass(THREADCLASS);
	assert(ThreadClass != 0);

	/* Create base group */
	standardGroup = (struct Hjava_lang_ThreadGroup*)execute_java_constructor(0, THREADGROUPCLASS, 0, "()V");
	assert(standardGroup != 0);
	
	/* Allocate a thread to be the main thread */
	createInitialThread("main");
	assert(currentThread);
	unhand(currentThread)->group = standardGroup;

	/* Attach thread to threadGroup */
	do_execute_java_method(0, (Hjava_lang_Object*)(unhand(currentThread)->group),
			       "add", "(Ljava/lang/Thread;)V", 0, 0, currentThread);

#if defined(GC_ENABLE)
	/* Start the GC daemons we need */
	finalman = createDaemon(&finaliserMan, "finaliser");
	garbageman = createDaemon(&gcMan, "gc");
	gc_mode = GC_ENABLED;
#endif

#if !defined(USE_NATIVE_THREADS)
	/* Plug in the alarm handler */
#if defined(SIGALRM)
	catchSignal(SIGALRM, alarmException);
#else
#error "No alarm signal support"
#endif
#endif
}

/*
 * Start a new thread running.
 */
void
startThread(Hjava_lang_Thread* tid)
{
	THREAD_CREATE(tid, &firstStartThread);
	
	unhand(currentThread)->state = 1; /* RUNNING */
}

/*
 * Create the initial thread with a given name.
 *  We should only ever call this once.
 */
static void
createInitialThread(char* nm)
{
	/* Allocate a thread to be the main thread */
	currentThread = (Hjava_lang_Thread*)newObject(ThreadClass);
	assert(currentThread != 0);

	unhand(currentThread)->name = makeJavaString(nm, strlen(nm));
	unhand(currentThread)->priority = java_lang_Thread_NORM_PRIORITY;
	unhand(currentThread)->threadQ = 0;
	/* unhand(currentThread)->single_step = 0; */
	unhand(currentThread)->daemon = 0;
	/* unhand(currentThread)->stillborn = 0; */
	unhand(currentThread)->target = 0;
	unhand(currentThread)->group = 0;
	unhand(currentThread)->state = 1; /* java.lang.Thread.CREATED */
	/* unhand(currentThread)->initialStackMemory = threadStackSize; */
	/* unhand(currentThread)->group = standardGroup; */

	THREAD_CREATEFIRST(currentThread);
}

/*
 * Start a daemon thread.
 */
static
Hjava_lang_Thread*
createDaemon(void* func, char* nm)
{
	Hjava_lang_Thread* tid;

	/* Keep daemon threads as root objects */
	tid = (Hjava_lang_Thread*)newObject(ThreadClass);
	assert(tid != 0);

DBG(DBG_THREADOP,
    	printf("createDaemon %p:%s\n", tid, nm);			)

	unhand(tid)->name = makeJavaString(nm, strlen(nm));
	unhand(tid)->priority = java_lang_Thread_MAX_PRIORITY;
	unhand(tid)->threadQ = 0;
	/* unhand(tid)->single_step = 0; */
	unhand(tid)->daemon = 1;
	/* unhand(tid)->stillborn = 0; */
	unhand(tid)->target = 0;
	/* unhand(tid)->initialStackMemory = threadStackSize; */
	unhand(tid)->group = 0;

	THREAD_CREATE(tid, func);

	return (tid);
}
