/*
 * thread-sun.c
 * SUN library-specific thread support.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

/*
 * NOTE: This file is #included by thread.c
 */

/*
 * Initialise threads.
 */
void
initThreads(void)
{
	/* Set default thread stack size if not set */
	if (threadStackSize == 0) {
		threadStackSize = THREADSTACKSIZE;
	}

	/* Get a handle on the thread and thread group classes */
	ThreadClass = lookupClass(THREADCLASS);
	assert(ThreadClass != 0);
	ThreadGroupClass = lookupClass(THREADGROUPCLASS);
	assert(ThreadGroupClass != 0);

	/* Create base group */
	standardGroup = (struct Hjava_lang_ThreadGroup*)newObject(ThreadGroupClass);
	assert(standardGroup != 0);
	unhand(standardGroup)->parent = 0;
	unhand(standardGroup)->name = makeJavaString("main", 4);
	unhand(standardGroup)->maxPriority = java_lang_Thread_MAX_PRIORITY;
	unhand(standardGroup)->destroyed = 0;
	unhand(standardGroup)->daemon = 0;
	unhand(standardGroup)->vmAllowSuspension = 0;
	unhand(standardGroup)->nthreads = 0;
	unhand(standardGroup)->threads = 0;
	unhand(standardGroup)->ngroups = 0;
	unhand(standardGroup)->groups = 0;

	/* Allocate a thread to be the main thread */
	createInitialThread("main");

#if defined(GC_ENABLE)
	/* Start the GC daemons we need */
	finalman = createDaemon(&finaliserMan, "finaliser");
	garbageman = createDaemon(&gcMan, "gc");
	gc_mode = GC_ENABLED;
#endif

#if !defined(USE_NATIVE_THREADS)
	/* Plug in the alarm handler */
#if defined(SIGALRM)
	catchSignal(SIGALRM, alarmException);
#else
#error "No alarm signal support"
#endif
#endif
}

/*
 * Start a new thread running.
 */
void
startThread(Hjava_lang_Thread* tid)
{
	THREAD_CREATE(tid, &firstStartThread);
}

/*
 * Create the initial thread with a given name.
 *  We should only ever call this once.
 */
static
void
createInitialThread(char* nm)
{
#ifdef DEBUG
	static int beenhere = 0;
	assert(beenhere == 0); beenhere++;
#endif

	/* Allocate a thread to be the main thread */
	currentThread = (Hjava_lang_Thread*)newObject(ThreadClass);
	assert(currentThread != 0);

	unhand(currentThread)->name = (HArrayOfChar*)makeJavaCharArray(nm, strlen(nm));
	unhand(currentThread)->priority = java_lang_Thread_NORM_PRIORITY;
	unhand(currentThread)->threadQ = 0;
	unhand(currentThread)->single_step = 0;
	unhand(currentThread)->daemon = 0;
	unhand(currentThread)->stillborn = 0;
	unhand(currentThread)->target = 0;
	unhand(currentThread)->initial_stack_memory = threadStackSize;
	unhand(currentThread)->group = standardGroup;

	THREAD_CREATEFIRST(currentThread);

	/* Attach thread to threadGroup */
	do_execute_java_method(0, (Hjava_lang_Object*)unhand(currentThread)->group, "add", "(Ljava/lang/Thread;)V", 0, 0, currentThread);
}

/*
 * Start a daemon thread.
 */
static
Hjava_lang_Thread*
createDaemon(void* func, char* nm)
{
	Hjava_lang_Thread* tid;

	/* Keep daemon threads as root objects */
	tid = (Hjava_lang_Thread*)newObject(ThreadClass);
	assert(tid != 0);

	DBG(DBG_THREADOP,
	    printf("createDaemon %s, tid=%p\n", nm, tid)
	    );

	unhand(tid)->name = (HArrayOfChar*)makeJavaCharArray(nm, strlen(nm));
	unhand(tid)->priority = java_lang_Thread_MAX_PRIORITY;
	unhand(tid)->threadQ = 0;
	unhand(tid)->single_step = 0;
	unhand(tid)->daemon = 1;
	unhand(tid)->stillborn = 0;
	unhand(tid)->target = 0;
	unhand(tid)->initial_stack_memory = threadStackSize;
	unhand(tid)->group = 0;

	THREAD_CREATE(tid, func);

	return (tid);
}
