/*
 * NativeLang-class.c
 *
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory (CSL).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 *      Author: Patrick Tullmann <tullmann@cs.utah.edu>, University of Utah CSL
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "../../../../kaffe/kaffevm/object.h"
#include "../../../../kaffe/kaffevm/constants.h"
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/itypes.h"
#include "../../../../kaffe/kaffevm/baseClasses.h"
#include "../../../../kaffe/kaffevm/classMethod.h"
#include "../../../../kaffe/kaffevm/file.h"
#include "../../../../kaffe/kaffevm/readClass.h"
#include "defs.h"
#include "java.lang.stubs/Throwable.h"
#include "java.lang.stubs/NativeLang.h"
#include <native.h>

HArrayOfObject* /* Class[] */
java_lang_NativeLang_getClassContext()
{
	unimp("Kore library function unimplemented");
	return 0;
}

struct Hjava_lang_Class*
java_lang_NativeLang_defineClass(struct Hjava_lang_ClassLoader *classLoader,
				 HArrayOfByte* data, jint offset, jint length)
{
	Hjava_lang_Class* classToBe;
	classFile classBytes;

	classBytes.base = &unhand(data)->body[offset];
	classBytes.buf  = classBytes.base;
	classBytes.size = length;

	classToBe = (Hjava_lang_Class*)newObject(&ClassClass);
	return readClass(classToBe, &classBytes, classLoader);
}

void
java_lang_NativeLang_resolveClass(struct Hjava_lang_Class* class)
{
	processClass(class, CSTATE_LINKED);
}

struct Hjava_lang_Class*
java_lang_NativeLang_findSystemClass(struct Hjava_lang_String* str)
{
	int len = javaStringLength(str);
	Utf8Const* c;
	char* name;
#if INTERN_UTF8CONSTS
	char buffer[100];
	if (len <= 100) {
		name = buffer;
	}
	else {
		name = malloc (len);
	}
#else
	c = malloc(sizeof(Utf8Const) + len + 1);
	name = c->data;
#endif
        javaString2CString(str, name, len+1);
	classname2pathname (name, name);
#if INTERN_UTF8CONSTS
	c = makeUtf8Const (name, len);
	if (name != buffer) {
		free(name);
	}
#else /* ! INTERN_UTF8CONSTS */
	c->length = len;
	c->hash = (uint16) hashUtf8String (name, len);
#endif /* ! INTERN_UTF8CONSTS */
	return (loadClass(c, 0));
}

