/*
 * NativeLang-system.c
 *
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory (CSL).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 *      Author: Patrick Tullmann <tullmann@cs.utah.edu>, University of Utah CSL
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#if defined(HAVE_SYS_UTSNAME_H)
#include <sys/utsname.h>
#endif
#if defined(HAVE_PWD_H)
#include <pwd.h>
#endif
#include "../../../../kaffe/kaffevm/object.h"
#include "../../../../kaffe/kaffevm/constants.h"
#include "../../../../kaffe/kaffevm/itypes.h"
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/access.h"
#include "../../../../kaffe/kaffevm/baseClasses.h"
#include "../../../../kaffe/kaffevm/classMethod.h"
#include "../../../../kaffe/kaffevm/support.h"
#include "system.h"
#include "defs.h"
#include "java.lang.stubs/Throwable.h"
#include "java.lang.stubs/NativeLang.h"
#include <native.h>

jlong
java_lang_NativeLang_currentTimeMillis()
{
	extern jlong currentTime(void);

	return currentTime();
}

void
java_lang_NativeLang_exit(jint status)
{
	exit(status);
}

struct Hjava_lang_Process* 
java_lang_NativeLang_exec(HArrayOfObject* /* String[] */cmdarray,
			  HArrayOfObject* /* String[] */envp)
{
	unimp("Kore library function unimplemented");
	return 0;
}

jbool
java_lang_NativeLang_load(struct Hjava_lang_String* filename)
{
	int len = javaStringLength(filename);
	char *name = malloc(len * sizeof(char));

	javaString2CString(filename, name, len+1);
	printf("Load: %s\n", name);
	unimp("Kore library function unimplemented");
	return 0;
}

jbool
java_lang_NativeLang_loadLibrary(struct Hjava_lang_String* libname)
{
	int len = javaStringLength(libname);
	char *name = malloc(len * sizeof(char));

	javaString2CString(libname, name, len+1);
	printf("LoadLibrary: %s\n", name);
	unimp("Kore library function unimplemented");
	return 0;
}

void
java_lang_NativeLang_gc()
{
	unimp("Kore library function unimplemented: gc");
}

struct Hjava_lang_String*
java_lang_NativeLang_defaultPropertiesFile()
{
	char *userhome = getenv("HOME"); /* XXX */
	char *propFileBase = "/.kore"; /* XXX */
	char *propFile;
	struct Hjava_lang_String *jstr;
	int length;
	
	if (!userhome)
		userhome = ".";

	length = strlen(userhome) + strlen(propFileBase);

	propFile = malloc(length);
	strcpy(propFile, userhome);
	strcat(propFile, propFileBase);

	jstr = makeJavaString(propFile, length);

	free(propFile);
	return jstr;
}

HArrayOfObject* /* String[] */
java_lang_NativeLang_getCommandLineProperties()
{
	extern userProperty* userProperties; /* Properties defined on command line */
	userProperty* prop;
	Hjava_lang_Object *strArray;
	int propCt = 0;
	int index;

	/* Count `em */
	for (prop = userProperties; prop != 0; prop = prop->next) {
		propCt += 2;
	}

	if (!propCt)
		return NULL; /* XXX */

	strArray = newRefArray(&StringClass, propCt);
		
	/* Stick em in. */
	index = 0;
	for (prop = userProperties; prop != 0; prop = prop->next) {
		assert(prop->key); /* XXX */
		OBJARRAY_DATA(strArray)[index++] =
			(Hjava_lang_Object*)makeJavaString(prop->key, strlen(prop->key));
		assert(prop->value); /* XXX */
		OBJARRAY_DATA(strArray)[index++] =
			(Hjava_lang_Object*)makeJavaString(prop->value, strlen(prop->value));
	}

	/* XXX release the userProperties. */

	return (HArrayOfObject*)strArray;
}

HArrayOfObject* /* String[] */
java_lang_NativeLang_getVMProperties()
{
	Hjava_lang_Object *strArray;
	char *java_home       = getenv(KAFFEHOME);
	char *java_class_path = getenv(KAFFECLASSPATH);
	char *os_name         = "Unknown";
	char *os_arch         = "Unknown";
	char *os_version      = "Unknown";
	char *user_dir_buf    = NULL;
	char *user_dir        = NULL;
	char *user_name       = "Unknown";
	char *user_home       = "Unknown";
	char *awt_toolkit     = NULL; /* XXX default value? */
	char *java_resources  = NULL;
	
	
	/*
	 * Set java.home, if the getenv() didn't work
	 */
	if (!java_home)
		java_home = ".";

	/*
	 * Get the default CLASSPATH, if the getenv() didn't work
	 */
#if !defined(DEFAULT_CLASSPATH)
#define DEFAULT_CLASSPATH "."
#endif
	if (!java_class_path)
		java_class_path = DEFAULT_CLASSPATH;
	
	/*
	 * Get OS properties: Name, architecture and version strings,
	 * All default to "Unknown", above.
	 */
	{
#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME) 
		struct utsname system;
		int r = uname(&system);
		assert(r >= 0); /* XXX */

		os_name = system.sysname;
		os_arch = system.machine;
		os_version = system.version;
#endif
	}
		
	/*
	 * Get the user's current directory (defaults to NULL, above.)
	 */
	user_dir_buf = malloc(sizeof(char) * MAXPATHLEN);
	assert(user_dir_buf); /* XXX */
#if defined(HAVE_GETCWD)
	user_dir = getcwd(user_dir_buf, MAXPATHLEN);
#elif defined(HAVE_GETWD)
	user_dir = getwd(user_dir_buf);
#endif
	if (!user_dir) 
		user_dir = ".";
	
	/*
	 * Get the user's name, defaults to 'Unknown', above
	 */
	{
#if defined(HAVE_PWD_H) && defined(HAVE_GETUID)
		struct passwd* pw;
		pw = getpwuid(getuid());
		if (pw)
			user_name = pw->pw_name;
#endif
	}

	/*
	 * Get the user's home directory, defaults to "Unknown", above
	 */
	{
#if defined(HAVE_PWD_H) && defined(HAVE_GETUID)
		struct passwd* pw;
		pw = getpwuid(getuid());
		if (pw)
			user_home = pw->pw_dir;
#endif
	}
	
	/*
	 * Figure out awt.toolkit, defaults to NULL, above.
	 */
#if defined(HAVE_PACKAGE_BISS_NET_COM)
	awt_toolkit = "biss.awt.kernel.Toolkit";
#elif defined(HAVE_PACKAGE_EPFL_CH)
	awt_toolkit = "kaffe.awt.simple.SimpleToolkit";
#endif

	/*
	 * Compute java.resources, an AWT lib directory of some sort.
	 * Defaults to NULL, above.
	 */
#if defined(HAVE_AWT_TOOLKIT_BISS) || defined(HAVE_AWT_TOOLKIT_SAWT)
	java_resources = malloc(sizeof(char) * MAXPATHLEN);
	strcpy(java_resources, java_home);
	strcat(java_resources, "/lib");
#endif
	
	{
		/* A struct for easing initialization---just simple ones */
		struct {
		      char *key;
		      char *value;
		} VMprops[] = {
			{ "file.separator",     "/" /* file_seperator*/ },
			{ "path.separator",     path_seperator },
			{ "line.separator",     line_seperator },
			{ "file.encoding.pkg",  "sun.io" },    /* Wha's this?  */
			{ "java.version",       kaffe_version },
			{ "java.vendor",        kaffe_vendor },
			{ "java.vendor.url",    kaffe_vendor_url },
			{ "java.class.version", kaffe_class_version },
			{ "java.home",          java_home },
			{ "java.class.path",    java_class_path },
			{ "os.name",            os_name },
			{ "os.arch",            os_arch },
			{ "os.version",         os_version },
			{ "user.dir",           user_dir },
			{ "user.name",          user_name },
			{ "user.home",          user_home },
			{ "awt.toolkit",        awt_toolkit },
			{ "java.resources",     java_resources }
		};
		int index, strindex;
		int numprops;
		
		/*
		 * Count the properties, and be sure to not count ones that
		 * have a NULL key or value.
		 */
		numprops=0;
		for (index = 0; index < (sizeof(VMprops) / sizeof(VMprops[0])); index++)
		{
			if (VMprops[index].key && VMprops[index].value)
				numprops++;
		}
		
		/*
		 * Allocate an array of strings, two strings for each property
		 */
		strArray = newRefArray(&StringClass, numprops*2);

		strindex = 0;
		for (index = 0; index < (sizeof(VMprops) / sizeof(VMprops[0])); index++)
		{
			char *key = VMprops[index].key;
			char *value = VMprops[index].value;
			
			/* Only add prop if its not null */
			if (key && value)
			{
				OBJARRAY_DATA(strArray)[strindex++] 
					= (Hjava_lang_Object*)makeJavaString(key, strlen(key));
				OBJARRAY_DATA(strArray)[strindex++] 
					= (Hjava_lang_Object*)makeJavaString(value, strlen(value));
			}
		}

		assert(strindex == (numprops * 2));
	}
	
	/*
	 * Free the buffers we allocated above.
	 */
	if (java_resources)
		free(java_resources);
	
	assert(user_dir_buf);
	free(user_dir_buf);

	return (HArrayOfObject*)strArray;
}

HArrayOfArray*
java_lang_NativeLang_getStackTrace0()
{
	extern Hjava_lang_Object* buildStackTrace(void* base);

	/* buildStackTrace actually returns an array of char arrays (eg char[][]). */
	return (HArrayOfArray*) buildStackTrace(0);
}


