/*
 * NativeLang-thread.c
 *
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory (CSL).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 *      Author: Patrick Tullmann <tullmann@cs.utah.edu>, University of Utah CSL
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "../../../../kaffe/kaffevm/object.h"
#include "../../../../kaffe/kaffevm/constants.h"
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/itypes.h"
#include "../../../../kaffe/kaffevm/baseClasses.h"
#include "../../../../kaffe/kaffevm/classMethod.h"
#include "../../../../kaffe/kaffevm/thread.h"
#include "NativeLang-thread.h"
#include "defs.h"
#include "java.lang.stubs/Throwable.h"
#include "java.lang.stubs/NativeLang.h"
#include <native.h>
#include "../../../../kaffe/kaffevm/debug.h"

struct Hjava_lang_Thread*
java_lang_NativeLang_currentThread(void)
{
	extern struct Hjava_lang_Thread *currentThread; /* kaffevm/thread.c, I hope */
	
	return currentThread;
}

void
java_lang_NativeLang_sleep(jlong millis, jint nanos)
{
	extern struct Hjava_lang_Thread *currentThread; /* kaffevm/thread.c, I hope */

	DBG(DBG_NATIVE_LANG_THREAD,
	    printf("sleep: Thread=%p, millis=%ld\n", currentThread, (long)millis)
	    );
	
	assert(unhand(currentThread)->state == thread_RUNNING);
	unhand(currentThread)->state = thread_ASLEEP;
	/* Implicitly operates on current Thread. */
	sleepThread(millis);
	unhand(currentThread)->state = thread_RUNNING;
}


char *
threadStateName(int st)
{
	static char unknown[64];
	switch(st)
	{
	case thread_CREATED:
		return "created";
	case thread_RUNNING:
		return "running";
	case thread_SUSPENDED:
		return "suspended";
	case thread_ASLEEP:
		return "asleep";
	case thread_INTERRUPTED:
		return "interrupted";
	case thread_STOPPED:
		return "stopped";
	case thread_DESTROYED:
		return "destroyed";
	case thread_WAITING:
		return "waiting";
	default:
		sprintf(unknown, "unknown (%d)", st);
		return unknown;
	}
}

void
java_lang_NativeLang_setThreadState(struct Hjava_lang_Thread* thread,
				    jint newState)
{
	int oldState = unhand(thread)->state;

	DBG(DBG_NATIVE_LANG_THREAD,
	    printf("setThreadState: Thread=%p, newState=%s, oldState=%s\n",
		   thread, threadStateName(newState),
		   threadStateName(oldState))
	    );

	/*
	 * XXX This code assumes that the caller is well behaved.
	 * (ie thread only in CREATED state once, at the beg., etc.)
	 */

	if (oldState == thread_CREATED)
	{
		assert(newState == thread_RUNNING);
		startThread(thread);
		unhand(thread)->state = newState;
		return;
	}
	

	switch (newState)
	{
	case thread_CREATED:
		SignalError(0,"java.lang.InternalError", 
			    "Thread state being set to 'CREATED'??!!\n");
		break;
	case thread_RUNNING:
		resumeThread(thread);
		break;
	case thread_SUSPENDED:
		suspendThread(thread);
		break;
	case thread_ASLEEP:
		SignalError(0,"java.lang.InternalError", 
			    "Thread state being set to 'SLEEP' in threadSetState()??!!\n");
		break;
	case thread_INTERRUPTED:
		SignalError(0,"java.lang.InternalError", 
			    "INTERRUPTED broken.\n");
		break;
	case thread_STOPPED:
		SignalError(0,"java.lang.InternalError", 
			    "Set thread STOPPED via stopThread method, not here.\n");
		break;
	case thread_DESTROYED:
		/* XXX */
		stopThread(thread);
		break;
	case thread_WAITING:
		SignalError(0,"java.lang.InternalError", 
			    "threadSetState(): WAITING broken.\n");
		break;
	default:
		SignalError(0,"java.lang.InternalError", 
			    "Unknown thread state requested.\n");
	}

	unhand(thread)->state = newState;
	return;
}

void java_lang_NativeLang_setThreadPriority(struct Hjava_lang_Thread* thread,
					    jint priority)
{
	setPriorityThread(thread, priority);
}

void java_lang_NativeLang_yield(void)
{
	yieldThread(); /* implicitly operates on current thread... */
}

void
java_lang_NativeLang_stopThread(struct Hjava_lang_Thread* thread,
				struct Hjava_lang_Throwable* error)
{
	DBG(DBG_NATIVE_LANG_THREAD,
	    printf("stopThread: Thread=%p, error=%p\n", thread, error)
	    );

	unhand(thread)->state = thread_STOPPED; /* XXX before or after stopThread()? */

	/* XXX deal with error object! */
	stopThread(thread);
}


jint
java_lang_NativeLang_countStackFrames(struct Hjava_lang_Thread* thread)
{
	unimp("Kore library function unimplemented");
	return 0;
}

