/*
 * NativeNet.c
 *
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory (CSL).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 *      Author: Patrick Tullmann <tullmann@cs.utah.edu>, University of Utah CSL
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include <netinet/in.h>
#include "config-io.h"
#include <netdb.h>
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/object.h"
#include "../../../../kaffe/kaffevm/itypes.h"
#include <sys/types.h> /* for size_t used in native.h */
#include "native.h"
#include "java.io.stubs/FileDescriptor.h"
#include "java.net.stubs/NativeNet.h"
#include "nets.h"

struct Hjava_lang_String*
java_net_NativeNet_getHostName(HArrayOfByte* addr)
{
	char name[MAXHOSTNAME];
	
	if (gethostname(name, MAXHOSTNAME-1) < 0)
		return (makeJavaString("localhost", strlen("localhost")));

	return (makeJavaString(name, strlen(name)));
}

HArrayOfByte*
java_net_NativeNet_getLocalHost()
{
	char name[MAXHOSTNAME];
	struct hostent* ent;
	Hjava_lang_Object* obj;

	if (gethostname(name, MAXHOSTNAME-1) < 0) 
		strcpy("localhost", name);
	
	ent = gethostbyname(name);
	if (ent == 0) {
		/* XXX */
		SignalError(0, "java.net.UnknownHostException", SYS_HERROR);
	}

	/* Copy in the network address */
	obj = AllocArray(sizeof(jint), TYPE_Byte);
	assert(obj != 0); /* XXX */
	*(jint*)ARRAY_DATA(obj) = *(jint*)ent->h_addr_list[0]; /* XXX */

	return ((HArrayOfByte*)obj);
}

HArrayOfArray* /* byte[][] */
java_net_NativeNet_getAllByName(struct Hjava_lang_String* host)
{
	unimp("Kore function not implemented");
	return 0;
}

void
java_net_NativeNet_socket(struct Hjava_io_FileDescriptor* out_fd,
			  jbool stream)
{
	unimp("Kore function not implemented");
}

void
java_net_NativeNet_bind_1(struct Hjava_io_FileDescriptor* fd,
			  HArrayOfByte* addr,
			  jint port)
{
	unimp("Kore function not implemented");
}

void
java_net_NativeNet_listen(struct Hjava_io_FileDescriptor* fd,
			  jint count)
{
	unimp("Kore function not implemented");
}

void
java_net_NativeNet_accept(struct Hjava_io_FileDescriptor* fd,
			  HArrayOfObject* results)
{
	unimp("Kore function not implemented");
}

jint
java_net_NativeNet_connect(struct Hjava_io_FileDescriptor* fd,
			   HArrayOfByte* address,
			   jint port)
{
	unimp("Kore function not implemented");
	return 0;
}

void
java_net_NativeNet_dgsocket(struct Hjava_io_FileDescriptor* out_fd)
{
	unimp("Kore function not implemented");
}

jint
java_net_NativeNet_bind_2(struct Hjava_io_FileDescriptor* fd,
			  jint port)
{
	unimp("Kore function not implemented");
	return 0;
}

void
java_net_NativeNet_recv(struct Hjava_io_FileDescriptor* fd,
			HArrayOfByte* buff,
			HArrayOfByte* addr,
			HArrayOfInt* other)
{
	unimp("Kore function not implemented");
}

void
java_net_NativeNet_send(struct Hjava_io_FileDescriptor* fd,
			HArrayOfByte* buff,
			jint length,
			HArrayOfByte* addr,
			jint port)
{
	unimp("Kore function not implemented");
}

void
java_net_NativeNet_close(struct Hjava_io_FileDescriptor* fd)
{
	unimp("Kore function not implemented");
}

jint
java_net_NativeNet_available(struct Hjava_io_FileDescriptor* fd)
{
	unimp("Kore function not implemented");
	return 0;
}

void
java_net_NativeNet_bind(struct Hjava_io_FileDescriptor* fd,
			HArrayOfByte* addr, jint off)
{
	unimp("Kore function not implemented");
}


